@@ -60,32 +60,37 @@ OPERATOR_MAP = {
 }
 
 
+def build_clause(query, attributes):
+    # Iterate through each query segment.
+    clause = None
+    last = None
+    for seg in query.segments:
+        # Get the attribute in question.
+        attribute = attributes[seg.path[0]]
+
+        # Replace the initial path segment with the expanded
+        # attribute path.
+        del seg.path[0]
+        seg.path[0:0] = attribute.path.split('.')
+
+        # Build the path from the segment.
+        path = '__'.join(seg.path) + OPERATOR_MAP[seg.operator]
+
+        # Construct a Q-object from the segment.
+        q = reduce(operator.or_, map(lambda x: Q((path, x)), seg.values))
+
+        # Combine the segment with the last.
+        clause = last.combinator(clause, q) if last is not None else q
+        last = seg
+
+    # Return the constructed clause.
+    return clause
+
+
 class ModelResource(object):
 
-    def filter(self, query, queryset):
-        # Iterate through each query segment.
-        clause = None
-        last = None
-        for seg in query.segments:
-            # Get the attribute in question.
-            attribute = self.attributes[seg.path[0]]
-
-            # Replace the initial path segment with the expanded
-            # attribute path.
-            del seg.path[0]
-            seg.path[0:0] = attribute.path.split('.')
-
-            # Build the path from the segment.
-            path = '__'.join(seg.path) + OPERATOR_MAP[seg.operator]
-
-            # Construct a Q-object from the segment.
-            q = reduce(operator.or_, map(lambda x: Q((path, x)), seg.values))
-
-            # Combine the segment with the last.
-            clause = last.combinator(clause, q) if last is not None else q
-            last = seg
-
-        # Filter by the constructed clause.
+    def filter(self, clause, queryset):
+        # Filter the queryset by the passed clause.
         return queryset.filter(clause).distinct()
 
     def read(self):
@@ -109,7 +114,12 @@ class ModelResource(object):
         # Determine if we need to filter the queryset in some way; and if so,
         # filter it.
         if query is not None:
-            queryset = self.filter(query, queryset)
+            clause = build_clause(query, self.attributes)
+            queryset = self.filter(clause, queryset)
+
+        # Filter the queryset by asserting authorization.
+        queryset = self.meta.authorization.filter(
+            self.request.user, 'read', self, queryset)
 
         if self.slug is not None:
             # Attempt to return just the single result we should have.
@@ -130,6 +140,11 @@ class ModelResource(object):
             if value is not None:
                 attribute.set(target, value)
 
+        # Ensure the user is authorized to perform this action.
+        authz = self.meta.authorization
+        if not authz.is_authorized(self.request.user, 'create', self, target):
+            authz.unauthorized()
+
         # Save the target.
         target.save()
 
@@ -142,6 +157,11 @@ class ModelResource(object):
             # Set each one on the target.
             attribute.set(target, data.get(name))
 
+        # Ensure the user is authorized to perform this action.
+        authz = self.meta.authorization
+        if not authz.is_authorized(self.request.user, 'update', self, target):
+            authz.unauthorized()
+
         # Save the target.
         target.save()
 
@@ -149,5 +169,10 @@ class ModelResource(object):
         # Grab the existing target.
         target = self.read()
 
+        # Ensure the user is authorized to perform this action.
+        authz = self.meta.authorization
+        if not authz.is_authorized(self.request.user, 'destroy', self, target):
+            authz.unauthorized()
+
         # Destroy the target.
         target.delete()
