@@ -2,6 +2,7 @@
 from __future__ import absolute_import, unicode_literals, division
 import six
 import operator
+from functools import partial
 from six.moves import map, reduce
 from armet.exceptions import ImproperlyConfigured
 from armet.query import parser, Query, QuerySegment, constants
@@ -29,6 +30,46 @@ OPERATOR_MAP = {
 }
 
 
+def build_segment(model, segment):
+    # Get the associated column for the initial path.
+    path = segment.path.pop(0)
+    col = model.__dict__[path]
+
+    # Resolve the inner-most path segment.
+    if segment.path:
+        return col.has(self.filter_segment(segment))
+
+    # Determine the operator.
+    op = OPERATOR_MAP[segment.operator]
+
+    # Apply the operator to the values and return the expression
+    return reduce(operator.or_, map(partial(op, col), segment.values))
+
+
+def build_clause(query, attributes, model):
+    # Iterate through each query segment.
+    clause = None
+    last = None
+    for seg in query.segments:
+        # Get the attribute in question.
+        attribute = attributes[seg.path[0]]
+
+        # Replace the initial path segment with the expanded
+        # attribute path.
+        del seg.path[0]
+        seg.path[0:0] = attribute.path.split('.')
+
+        # Construct the clause from the segment.
+        q = build_segment(model, seg)
+
+        # Combine the segment with the last.
+        clause = last.combinator(clause, q) if last is not None else q
+        last = seg
+
+    # Return the constructed clause.
+    return clause
+
+
 class ModelResource(object):
     """Specializes the RESTFul model resource protocol for SQLAlchemy.
 
@@ -41,42 +82,8 @@ class ModelResource(object):
         # Establish a session using our session type object.
         self.session = self.meta.Session()
 
-    def filter_segment(self, segment):
-        # Get the associated column for the initial path.
-        path = segment.path.pop(0)
-        col = self.meta.model.__dict__[path]
-
-        # Resolve the inner-most path segment.
-        if segment.path:
-            return col.has(self.filter_segment(segment))
-
-        # Determine the operator.
-        op = OPERATOR_MAP[segment.operator]
-
-        # Apply the operator to the values and return the expression
-        return reduce(operator.or_, map(lambda x: op(col, x), segment.values))
-
-    def filter(self, query, queryset):
-        # Iterate through each query segment.
-        clause = None
-        last = None
-        for seg in query.segments:
-            # Get the attribute in question.
-            attribute = self.attributes[seg.path[0]]
-
-            # Replace the initial path segment with the expanded
-            # attribute path.
-            del seg.path[0]
-            seg.path[0:0] = attribute.path.split('.')
-
-            # Construct the clause from the segment.
-            q = self.filter_segment(seg)
-
-            # Combine the segment with the last.
-            clause = last.combinator(clause, q) if last is not None else q
-            last = seg
-
-        # Filter by the constructed clause.
+    def filter(self, clause, queryset):
+        # Filter the queryset by the passed clause.
         return queryset.filter(clause).distinct()
 
     def read(self):
@@ -99,8 +106,14 @@ class ModelResource(object):
 
         # Determine if we need to filter the queryset in some way; and if so,
         # filter it.
+        clause = None
         if query is not None:
-            queryset = self.filter(query, queryset)
+            clause = build_clause(query, self.attributes, self.meta.model)
+            queryset = self.filter(clause, queryset)
+
+        # Filter the queryset by asserting authorization.
+        queryset = self.meta.authorization.filter(
+            self.request.user, 'read', self, queryset)
 
         # Return the queryset.
         return queryset.all() if self.slug is None else queryset.first()
@@ -116,6 +129,11 @@ class ModelResource(object):
             if value is not None:
                 attribute.set(target, value)
 
+        # Ensure the user is authorized to perform this action.
+        authz = self.meta.authorization
+        if not authz.is_authorized(self.request.user, 'create', self, target):
+            authz.unauthorized()
+
         # Add the target to the session.
         self.session.add(target)
 
@@ -131,6 +149,11 @@ class ModelResource(object):
             # Set each one on the target.
             attribute.set(target, data.get(name))
 
+        # Ensure the user is authorized to perform this action.
+        authz = self.meta.authorization
+        if not authz.is_authorized(self.request.user, 'update', self, target):
+            authz.unauthorized()
+
         # Commit the session.
         self.session.commit()
 
@@ -138,6 +161,11 @@ class ModelResource(object):
         # Grab the existing target.
         target = self.read()
 
+        # Ensure the user is authorized to perform this action.
+        authz = self.meta.authorization
+        if not authz.is_authorized(self.request.user, 'destroy', self, target):
+            authz.unauthorized()
+
         # Remove the object from the session.
         self.session.delete(target)
 
